@@ -7,42 +7,47 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 #include <glib-object.h>
+#include <cairo/cairo.h>
+#include <math.h>
+#include <string.h>
 
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
 
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-
 #include "gtkgridboard.h"
 
+#define DO_PROFILE
+
+#ifdef DO_PROFILE
+#include <sys/time.h>
+
+/* A temporary profiling counter. Only supports one instance.*/
+long int counter;
+#endif
+
 static GtkWidgetClass *parent_class=NULL;
 static guint gridboard_signals[LAST_SIGNAL] = { 0 };
 
-/* Save the flip pixmaps timeout so we can kill it at the games end. */
-guint timeoutid = 0;
-
 /* prototypes for private functions */
-static gint ** make_array(int width, int height); 
-static void gtk_gridboard_draw_box(GtkGridBoard *widget, gint x, gint y); 
-static void gtk_gridboard_draw_box_with_gc(GtkGridBoard * widget, GdkGC * gc, gint x, gint y);
-static void gtk_gridboard_load_pixmaps (GtkGridBoard * gridboard);
+static void gtk_gridboard_repaint(GtkGridBoard * gridboard);
+static gint ** make_array(int width, int height, int fill); 
+static void gtk_gridboard_load_tiles (GtkGridBoard * gridboard);
 static gint get_pixmap_num(int piece);
-static gint gtk_gridboard_button_press(GtkWidget *widget, GdkEventButton *event) ;
-static gint gtk_gridboard_expose(GtkWidget * widget, GdkEventExpose * event);
+static gint gtk_gridboard_button_press(GtkWidget *widget, 
+				       GdkEventButton *event) ;
+static gboolean gtk_gridboard_expose(GtkWidget * widget, 
+				     GdkEventExpose * event);
+static void gtk_gridboard_size_allocate(GtkWidget * widget, 
+					GtkAllocation * allocation);
 static void gtk_gridboard_class_init(GtkGridBoardClass * class);
 static void gtk_gridboard_finalize(GObject * object);
 static void gtk_gridboard_init(GtkGridBoard * gridboard);
 static void gtk_gridboard_realize(GtkWidget * widget);
-static void gtk_gridboard_size_allocate(GtkWidget * widget, GtkAllocation * allocation);
 static void gtk_gridboard_size_request(GtkWidget * widget, GtkRequisition * req);
-static void gtk_gridboard_draw_pixmap(GtkGridBoard * gridboard, gint which, gint x, gint y);
+static void gtk_gridboard_draw_pixmap(GtkGridBoard * gridboard, gdouble phase, 
+				      gdouble x, gdouble y, 
+				      cairo_pattern_t *pattern);
 static gint gtk_gridboard_flip_pixmaps(gpointer data);
 
 /* init functions */
@@ -60,8 +65,8 @@ static void gtk_gridboard_class_init(GtkGridBoardClass * class) {
         object_class->finalize = gtk_gridboard_finalize;
         widget_class->realize = gtk_gridboard_realize;
         widget_class->expose_event = gtk_gridboard_expose;
-        widget_class->size_request = gtk_gridboard_size_request;
-        widget_class->size_allocate = gtk_gridboard_size_allocate;
+	widget_class->size_allocate = gtk_gridboard_size_allocate;
+	widget_class->size_request = gtk_gridboard_size_request;
         widget_class->button_press_event = gtk_gridboard_button_press;
         
         gridboard_signals[BOXCLICKED] = g_signal_new ("boxclicked",
@@ -74,15 +79,18 @@ static void gtk_gridboard_class_init(GtkGridBoardClass * class) {
 }
 
 static void gtk_gridboard_init(GtkGridBoard * gridboard) {
+        gridboard->backing_store = NULL;
+	gridboard->tiles_pixbuf = NULL;
+	gridboard->tiles_scale = 1;
         gridboard->tileset=NULL;
-        gridboard->showgrid=FALSE;
-        gridboard->policy=GTK_UPDATE_CONTINUOUS;
         gridboard->pixmaps=NULL;
+	gridboard->timeoutid = 0;
         gridboard->tileheight=TILEHEIGHT;
         gridboard->tilewidth=TILEWIDTH;
         gridboard->width=0;
         gridboard->height=0;
 	gridboard->statelist=NULL;
+	gridboard->themesurface = NULL;
 }
 
 GtkWidget * gtk_gridboard_new(gint width, gint height, char * tileset) {
@@ -92,11 +100,12 @@ GtkWidget * gtk_gridboard_new(gint width, gint height, char * tileset) {
         gridboard->height=height;
         gridboard->tileset=tileset;
 
-        gridboard->board=make_array(width, height);
-        gridboard->pixmaps=make_array(width, height);
-        gridboard->selected=make_array(width, height);
-
-        gtk_gridboard_load_pixmaps(gridboard);
+        gridboard->board    = make_array (width, height, 0);
+        gridboard->pixmaps  = make_array (width, height, 0);
+        gridboard->selected = make_array (width, height, SELECTED_NONE);
+	gridboard->changed  = make_array (width, height, TRUE);
+	
+        gtk_gridboard_load_tiles(gridboard);
         gtk_gridboard_set_animate(gridboard, TRUE);
         gtk_gridboard_set_visibility(gridboard, TRUE);
         
@@ -141,8 +150,8 @@ static gint gtk_gridboard_button_press(GtkWidget *widget,
    
    gridboard = GTK_GRIDBOARD (widget);
 
-   x = event->x / gridboard->tilewidth;
-   y = event->y / gridboard->tileheight;
+   x = event->x * gridboard->width / widget->allocation.width;
+   y = event->y * gridboard->height / widget->allocation.height;
 
    x = CLAMP (x, 0, gridboard->width - 1);
    y = CLAMP (y, 0, gridboard->height - 1);
@@ -156,18 +165,40 @@ static gint gtk_gridboard_button_press(GtkWidget *widget,
 static gboolean gtk_gridboard_expose(GtkWidget *widget,
 				     GdkEventExpose * event)
  {
-        g_return_val_if_fail (GTK_IS_GRIDBOARD(widget), FALSE);
+        GtkGridBoard *gridboard;
 
-        /* multiple exposures, do only the last one */
-        if (event->count>0) return FALSE;
+	if (!GTK_WIDGET_DRAWABLE (widget))
+	  return FALSE;
 
-	/* FIXME: We should only redraw the bit we need to. */
+	gridboard = GTK_GRIDBOARD (widget);
 
-        gtk_gridboard_paint(GTK_GRIDBOARD (widget));
+	gdk_draw_drawable (widget->window, widget->style->black_gc,
+			   gridboard->backing_store, event->area.x,
+			   event->area.y, event->area.x, event->area.y,
+			   event->area.width, event->area.height);
 
         return FALSE;
 }
 
+static void gtk_gridboard_size_allocate (GtkWidget * widget,
+					 GtkAllocation *allocation)
+{
+  GtkGridBoard *gridboard = GTK_GRIDBOARD (widget);
+
+  widget->allocation = *allocation;
+ 
+  if (gridboard->backing_store != NULL) {
+    g_object_unref (gridboard->backing_store);
+    gridboard->backing_store = NULL;
+  }
+
+  if (GTK_WIDGET_REALIZED (widget)) {
+    gdk_window_move_resize (widget->window, allocation->x, allocation->y,
+			    allocation->width, allocation->height);
+    gtk_gridboard_paint (gridboard);
+  }
+}
+
 static void gtk_gridboard_realize(GtkWidget * widget) 
 {
         GdkWindowAttr attributes;
@@ -198,6 +229,8 @@ static void gtk_gridboard_realize(GtkWidget * widget)
         gdk_window_set_user_data (widget->window, widget);
 
         gtk_style_set_background (widget->style, widget->window, GTK_STATE_ACTIVE);
+	
+	gtk_gridboard_paint (GTK_GRIDBOARD (widget));
 }
 
 static void gtk_gridboard_finalize(GObject * object) {
@@ -208,9 +241,11 @@ static void gtk_gridboard_finalize(GObject * object) {
 
         gridboard = GTK_GRIDBOARD(object);
 
-        g_free(gridboard->board);
-        g_free(gridboard->pixmaps);
-        g_free(gridboard->selected);
+	g_object_unref (gridboard->backing_store);
+        g_free (gridboard->board);
+        g_free (gridboard->pixmaps);
+        g_free (gridboard->selected);
+	g_free (gridboard->changed);
 
         if (G_OBJECT_CLASS (parent_class)->finalize)
                 (* G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -227,78 +262,171 @@ static void gtk_gridboard_size_request(GtkWidget * widget, GtkRequisition * req)
         req->height=gridboard->height*gridboard->tileheight + 1;
 }
 
-/* this isn't really used because the window is not resizable */
-static void gtk_gridboard_size_allocate(GtkWidget *widget,
-					GtkAllocation * allocation) 
-{
-        GtkGridBoard *gridboard; 
-
-        g_return_if_fail(GTK_IS_GRIDBOARD(widget));
-  
-	gridboard = GTK_GRIDBOARD (widget);
-
-	widget->allocation = *allocation;
-
-        if (GTK_WIDGET_REALIZED (widget)) {
-                gdk_window_move_resize (widget->window,
-                        allocation->x, 
-                        allocation->y,
-                        allocation->width, 
-                        allocation->height);
-        }
-}
-
-
 /* drawing functions */
+
 static gint get_pixmap_num(int piece) {
         if (piece==BLACK) return BLACK_PIXMAP;
         if (piece==WHITE) return WHITE_PIXMAP;
         return EMPTY_PIXMAP;
 }
 
-static void gtk_gridboard_redraw_grid (GtkGridBoard *gridboard)
+void gtk_gridboard_paint (GtkGridBoard *gridboard)
 {
-  int x, y;
+  GtkWidget *widget;
 
+  g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
+  
+  widget = GTK_WIDGET (gridboard);
+  g_return_if_fail (GTK_WIDGET_REALIZED (widget));
+
+  gtk_widget_queue_draw (widget);
+
+  gtk_gridboard_repaint (gridboard);
+}
+
+#define COIN_RADIUS 0.4
+
+static void gtk_gridboard_repaint(GtkGridBoard * gridboard) {
+  int x, y;
+  double phase;
+  GtkWidget *widget;
+  gdouble scale;
+  cairo_t *themecx;
+  gint w, h;
+  
+#ifdef DO_PROFILE
+  struct timeval tv;
+#endif
+  
+  g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
+  if (!gridboard->visibility) return;
+  
+  widget = GTK_WIDGET (gridboard);
+  g_return_if_fail (GTK_WIDGET_REALIZED (widget));
+
+  if (gridboard->backing_store == NULL) {
+    gridboard->backing_store = gdk_pixmap_new (widget->window,
+					       widget->allocation.width,
+					       widget->allocation.height, 
+					       -1);
+    for (x=0; x<gridboard->width; x++)
+      for (y=0; y<gridboard->height; y++)
+	gridboard->changed[x][y] = TRUE;
+  }
+  
+  gridboard->cx = gdk_cairo_create (gridboard->backing_store);
+  cairo_scale (gridboard->cx, 
+	       1.0*widget->allocation.width/gridboard->width,
+	       1.0*widget->allocation.height/gridboard->height);
+  
+  /* Make a new cairo surface with the background drawn on it. */
+  /* FIXME: Should be in the realize callback. */
+  /* FIXME: This doesn't allow for the theme to change at all :( */
+  if (gridboard->themesurface == NULL) {
+#ifdef DO_PROFILE
+    gettimeofday (&tv, NULL);
+    counter = tv.tv_sec * 1000000 + tv.tv_usec;
+#endif
+    w = gdk_pixbuf_get_width (gridboard->tiles_pixbuf);
+    h = gdk_pixbuf_get_height (gridboard->tiles_pixbuf);
+    
+    gridboard->themesurface = 
+      cairo_surface_create_similar (cairo_get_target (gridboard->cx), 
+				    CAIRO_CONTENT_COLOR_ALPHA,
+				    w, h);
+    themecx = cairo_create (gridboard->themesurface);
+    gdk_cairo_set_source_pixbuf (themecx, gridboard->tiles_pixbuf,
+				 0.0, 0.0);
+    cairo_paint (themecx);
+#ifdef DO_PROFILE
+    gettimeofday (&tv, NULL);
+    g_print ("[%ld]\n", tv.tv_sec * 1000000 + tv.tv_usec - counter);
+#endif
+  }
+  
+#ifdef DO_PROFILE
+  gettimeofday (&tv, NULL);
+  counter = tv.tv_sec * 1000000 + tv.tv_usec;
+#endif
+    
+  scale = gridboard->tiles_scale;
+  
   for (x=0; x<gridboard->height; x++) {
     for (y=0; y<gridboard->width; y++) {
-      gtk_gridboard_draw_box(gridboard, x, y);
+      if (gridboard->changed[x][y]) {
+	/* Fill in the background. */
+	cairo_save (gridboard->cx);
+	cairo_rectangle (gridboard->cx, x, y, 1, 1);
+	cairo_clip_preserve (gridboard->cx);
+	/* The background. */
+	cairo_set_source_rgba (gridboard->cx, 0.43, 0.55, 0.65, 1.0);
+	cairo_fill_preserve (gridboard->cx);
+	/* The grid. */
+	cairo_set_line_width (gridboard->cx, 0.02);
+	cairo_set_source_rgba (gridboard->cx, 0.0, 0.0, 0.0, 1.0);
+	cairo_stroke (gridboard->cx);
+	if (gridboard->board[x][y] != EMPTY) {
+	  phase = gridboard->pixmaps[x][y];
+	  phase = (phase - 1.0)/30.0;
+	  cairo_save (gridboard->cx);
+	  gtk_gridboard_draw_pixmap(gridboard, phase, 
+				    x, y, 
+				    NULL);
+	  cairo_restore (gridboard->cx);
+	} else if (gridboard->selected[x][y]) {
+	  cairo_save (gridboard->cx);
+	  cairo_set_source_rgba (gridboard->cx, 1.0, 1.0, 1.0, 0.3);
+	  cairo_arc (gridboard->cx, x + 0.5, y + 0.5, COIN_RADIUS, 0,
+		     2*G_PI);
+	  cairo_fill (gridboard->cx);
+	  cairo_restore (gridboard->cx);
+	}
+	gridboard->changed[x][y] = FALSE;
+	cairo_restore (gridboard->cx);
+      }
     }
   }
+
+  cairo_destroy (gridboard->cx);
+  
+#ifdef DO_PROFILE
+  gettimeofday (&tv, NULL);
+  g_print ("%ld\n", tv.tv_sec * 1000000 + tv.tv_usec - counter);
+#endif
 }
 
-void gtk_gridboard_paint(GtkGridBoard * gridboard) {
-        int x, y;
-        int piecepic;
 
-	if (!gridboard->visibility) return;
+static void gtk_gridboard_draw_pixmap(GtkGridBoard * gridboard, gdouble phase, 
+				      gdouble x, gdouble y, 
+				      cairo_pattern_t *pattern) 
+{
+  x = x + 0.5;
+  y = y + 0.5;
 
-        for (x=0; x<gridboard->height; x++) {
-                for (y=0; y<gridboard->width; y++) {
-                        piecepic=get_pixmap_num(gridboard->board[x][y]);
-                        gtk_gridboard_draw_pixmap(gridboard, piecepic, x, y);
-                }
-        }
-	gtk_gridboard_redraw_grid (gridboard);
- }
+  /* We special-case the two "pure" ends of the flip to avoid
+   * floating point errors in calculating the phase. */
+  if (phase > 0.5) {
+    cairo_set_source_rgb (gridboard->cx, 1.0, 1.0, 1.0);
+  } else {
+    cairo_set_source_rgb (gridboard->cx, 0.0, 0.0, 0.0);
+  }
 
+  if ((phase > 0.98) || (phase < 0.02)) {
+    cairo_arc (gridboard->cx, x, y, COIN_RADIUS, 0, 2*G_PI);
+    cairo_fill (gridboard->cx);
+  } else {
+    cairo_save (gridboard->cx);
+    cairo_translate (gridboard->cx, x, y);
+    cairo_scale (gridboard->cx, cos (phase*G_PI), 1.0);
+    cairo_arc (gridboard->cx, 0, 0, COIN_RADIUS, 0, 2*G_PI);
+    cairo_fill (gridboard->cx);
+    cairo_restore (gridboard->cx);
+  }
 
-static void gtk_gridboard_draw_pixmap(GtkGridBoard * gridboard, gint which, gint
- x, gint y) {
-        GtkWidget * widget=GTK_WIDGET(gridboard);
-        
-	if (!gridboard->visibility) return;
-
-        gdk_draw_drawable (widget->window,
-                widget->style->fg_gc[widget->state],
-                gridboard->tiles_pixmap,
-                (which % 8) * TILEWIDTH, (which / 8) * TILEHEIGHT,
-                x * TILEWIDTH, y * TILEHEIGHT, TILEWIDTH, TILEHEIGHT);
-
-        /* draw lines around x, y */
-        gtk_gridboard_draw_box(gridboard, x, y);
 }
 
+#undef COIN_RADIUS
+
 void gtk_gridboard_set_selection(GtkGridBoard * gridboard, gint type, 
 				 gint x, gint y) 
 {
@@ -308,49 +436,8 @@ void gtk_gridboard_set_selection(GtkGridBoard * gridboard, gint type,
 	g_return_if_fail (x < gridboard->width);
 	g_return_if_fail (y < gridboard->height);
 
-        gridboard->selected[x][y]=type;
-        gtk_gridboard_draw_box(gridboard, x, y);
-}
-
-static void gtk_gridboard_draw_box(GtkGridBoard *gridboard, gint x, gint y) {
-        GtkWidget *widget;
-        GdkGC * gc;
-        int selected;
-        static GdkColor selectcolor={12345, 30208, 41216, 33792};
-
-	if (!gridboard->visibility) return;
-        
-	widget = GTK_WIDGET (gridboard);
-
-        selected=gridboard->selected[x][y];
-        
-        if (selected==SELECTED_NONE) {
-                if (!gridboard->showgrid) {
-        		return;
-                } else {
-                        gc=widget->style->black_gc;
-                }
-        } else if (selected==SELECTED_A) {
-                gc=widget->style->white_gc;
-        } else if (selected==SELECTED_B) {
-                gc=gdk_gc_new(widget->window);
-                gdk_gc_set_foreground(gc, &selectcolor);
-        } else {
-		return;
-	}
-        gtk_gridboard_draw_box_with_gc(gridboard, gc, x, y);
-}
-
-static void gtk_gridboard_draw_box_with_gc(GtkGridBoard * gridboard, 
-					   GdkGC * gc, gint x, gint y) 
-{
-        GtkWidget * widget;
-
-	widget = GTK_WIDGET (gridboard);
-
-        gdk_draw_rectangle(widget->window, gc, FALSE,
-                x*gridboard->tilewidth, y*gridboard->tileheight,
-                gridboard->tilewidth, gridboard->tileheight);
+        gridboard->selected[x][y] = type;
+	gridboard->changed[x][y]  = TRUE;
 }
 
 static gint gtk_gridboard_flip_pixmaps(gpointer data) 
@@ -362,39 +449,47 @@ static gint gtk_gridboard_flip_pixmaps(gpointer data)
         gint pcepm; /* this pixmap should be there */
         gint flipped_tiles;
 
+	if (!GTK_WIDGET_DRAWABLE (GTK_WIDGET (gridboard))) {
+	  return gridboard->animate;
+	}
+
+	flipped_tiles=0;
         for (x=0; x<gridboard->width; x++) {
                 for (y=0; y<gridboard->height; y++) {
+        		/* curpm is the current pixmap being shown.
+			 * pcepm is the target pixmap. We step from
+			 * one to the other during the animation. */
                         curpm=gridboard->pixmaps[x][y];
                         pcepm=get_pixmap_num(gridboard->board[x][y]);
-                        flipped_tiles=0;
-                        if (curpm==pcepm) {
-                                /* nothing needs to be flipped */
+                        if (curpm == pcepm) {
+                                /* Nothing needs to be flipped */
                                 continue;
-                        } else if ((curpm==EMPTY_PIXMAP)||(pcepm==EMPTY_PIXMAP)) {
-                                /* don't animate empty squares */
+                        } else if ((curpm == EMPTY_PIXMAP) ||
+				   (pcepm == EMPTY_PIXMAP)) {
+                                /* Don't animate empty squares */
                                 gridboard->pixmaps[x][y]=pcepm;
                                 flipped_tiles++;
-                        } else if (curpm<pcepm) {
+				gridboard->changed[x][y] = TRUE;
+                        } else if (curpm < pcepm) {
                                 gridboard->pixmaps[x][y]++;
                                 flipped_tiles++;
-                        } else if (curpm>pcepm) {
+				gridboard->changed[x][y] = TRUE;
+                        } else { /* curpm > pcepm */
                                 gridboard->pixmaps[x][y]--;
                                 flipped_tiles++;
-                        }
-                        if (flipped_tiles) {
-                                gtk_gridboard_draw_pixmap(
-                                                gridboard,
-                                                gridboard->pixmaps[x][y],
-                                                x, y);
+				gridboard->changed[x][y] = TRUE;
                         }
                 }
         }
 
+	if (flipped_tiles) 
+	  gtk_gridboard_paint (gridboard);
+
         /* destroy the timeout if animate is false */
         return gridboard->animate;
 }
 
-static void gtk_gridboard_load_pixmaps (GtkGridBoard * gridboard) {
+static void gtk_gridboard_load_tiles (GtkGridBoard * gridboard) {
         gchar *filepath=gridboard->tileset;
         GdkPixbuf *image;
 
@@ -410,32 +505,31 @@ static void gtk_gridboard_load_pixmaps (GtkGridBoard * gridboard) {
                 exit(1);
         }
 
-        /* what happens here ? */
-        gdk_pixbuf_render_pixmap_and_mask_for_colormap (image, 
-							gdk_colormap_get_system (),
-							&(gridboard->tiles_pixmap), 
-							NULL, 127);
-  
-        gdk_pixbuf_unref (image);
+	if (gridboard->tiles_pixbuf != NULL)
+	  g_object_unref (gridboard->tiles_pixbuf);
+
+	gridboard->tiles_pixbuf = image;
+	gridboard->tiles_scale = gdk_pixbuf_get_height (image);
 }
 
 
 /* helper functions */
 
-/* allocates memory for the array */
-static gint ** make_array(int width, int height) {
-        gint ** result;
-        int valarraysiz, ptrarraysiz;
-        int i;
-        
-        valarraysiz=width*height;       /* size of actual array */
-        ptrarraysiz=width;              /* size of pointer array */
-        result=(gint **)g_malloc(ptrarraysiz*sizeof(gint *)+valarraysiz*sizeof(gint));
-        memset(result, 0, ptrarraysiz*sizeof(gint *)+valarraysiz*sizeof(gint));
-        for (i=0; i<width; i++) {
-                result[i]=(gint *)(result+ptrarraysiz)+i*height;
-        }
-        return result;
+/* Allocates memory for an array and fills it with the byte fill. */
+static gint ** make_array(int width, int height, int fill) 
+{
+  gint ** result;
+  int valarraysiz, ptrarraysiz;
+  int i;
+  
+  valarraysiz = width*height;       /* size of actual array */
+  ptrarraysiz = width;              /* size of pointer array */
+  result = (gint **) g_malloc (ptrarraysiz*sizeof(gint *)+valarraysiz*sizeof(gint));
+  for (i=0; i<width; i++) {
+    result[i] = (gint *)(result+ptrarraysiz)+i*height;
+    memset (result[i], fill, sizeof (gint)*height);
+  }
+  return result;
 }
 
 int gtk_gridboard_count_pieces(GtkGridBoard * gridboard, int piece) {
@@ -489,24 +583,17 @@ void g_cclosure_user_marshal_VOID__INT_INT (GClosure     *closure,
 
 /* option setting functions */
 
-void gtk_gridboard_set_show_grid(GtkGridBoard *gridboard, gboolean showgrid) {
-        g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
-
-        gridboard->showgrid=showgrid;
-        gtk_gridboard_paint(gridboard);
-}
-
 void gtk_gridboard_set_animate(GtkGridBoard *gridboard, gboolean animate) {
         g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
 
         gridboard->animate=animate;
         if (animate) {
-	  if (timeoutid)
+	  if (gridboard->timeoutid)
 	    return;
-	  timeoutid = g_timeout_add(PIXMAP_FLIP_DELAY, 
-				    gtk_gridboard_flip_pixmaps,
-				    (gpointer) gridboard);
-        } else timeoutid = 0;
+	  gridboard->timeoutid = g_timeout_add(PIXMAP_FLIP_DELAY, 
+					       gtk_gridboard_flip_pixmaps,
+					       (gpointer) gridboard);
+        } else gridboard->timeoutid = 0;
 }
 
 void gtk_gridboard_set_visibility(GtkGridBoard *gridboard, gboolean visibility) {
@@ -519,8 +606,7 @@ void gtk_gridboard_set_tileset(GtkGridBoard *gridboard, gchar * tileset) {
         g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
 
         gridboard->tileset=tileset;
-        gtk_gridboard_load_pixmaps(gridboard);
-        gtk_gridboard_paint(gridboard);
+        gtk_gridboard_load_tiles(gridboard);
 }
         
 /* board altering functions */
@@ -529,12 +615,15 @@ void gtk_gridboard_set_piece(GtkGridBoard *gridboard, int x, int y,
 {
         g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
 
+	if (gridboard->board[x][y] == piece)
+	  return;
+
         gridboard->board[x][y]=piece;
-        /* draw updated piece */
-        if (!gridboard->animate) {
-                gtk_gridboard_draw_pixmap(gridboard, get_pixmap_num(piece), x, y
-);
-        }
+	
+	if (!gridboard->animate) {
+	  gridboard->pixmaps[x][y] = get_pixmap_num (gridboard->board[x][y]);
+	  gridboard->changed[x][y] = TRUE;
+	}
 }
 
 int gtk_gridboard_get_piece(GtkGridBoard *gridboard, int x, int y) 
@@ -552,10 +641,12 @@ void gtk_gridboard_clear_selections(GtkGridBoard *gridboard)
 
 	for (x=0; x<gridboard->width; x++) {
 		for (y=0; y<gridboard->height; y++) {
-			gridboard->selected[x][y]=SELECTED_NONE;
+		  if (gridboard->selected[x][y] != SELECTED_NONE) {
+		    gridboard->selected[x][y] = SELECTED_NONE;
+		    gridboard->changed[x][y] = TRUE;
+		  }
 		}
 	}
-	gtk_gridboard_paint (gridboard);
 }
 
 void gtk_gridboard_clear_pieces(GtkGridBoard *gridboard) 
@@ -566,24 +657,10 @@ void gtk_gridboard_clear_pieces(GtkGridBoard *gridboard)
 
 	for (x=0; x<gridboard->width; x++) {
 		for (y=0; y<gridboard->height; y++) {
-			gridboard->board[x][y]=EMPTY;
+			gridboard->board[x][y] = EMPTY;
+			gridboard->changed[x][y] = TRUE;
 		}
 	}
-	gtk_gridboard_paint(gridboard);
-}
-
-void gtk_gridboard_clear_pixmaps(GtkGridBoard *gridboard) 
-{
-	int x, y;
-
-        g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
-
-	for (x=0; x<gridboard->width; x++) {
-		for (y=0; y<gridboard->height; y++) {
-			gridboard->pixmaps[x][y]=get_pixmap_num(EMPTY);
-		}
-	}
-	gtk_gridboard_paint(gridboard);
 }
 
 void gtk_gridboard_clear(GtkGridBoard *gridboard) 
@@ -592,7 +669,6 @@ void gtk_gridboard_clear(GtkGridBoard *gridboard)
 
 	gtk_gridboard_clear_selections(gridboard);
 	gtk_gridboard_clear_pieces(gridboard);
-	gtk_gridboard_clear_pixmaps(gridboard);
 	gtk_gridboard_clear_states(gridboard);
 }
 
@@ -613,7 +689,7 @@ void gtk_gridboard_save_state(GtkGridBoard *gridboard, gpointer data)
 
 	newstate = g_malloc(sizeof(StateList));
 
-	newstate->board=make_array(gridboard->width, gridboard->height);
+	newstate->board = make_array (gridboard->width, gridboard->height, 0);
 	for (x=0; x<gridboard->width; x++) {
 		for (y=0; y<gridboard->height; y++) {
 			newstate->board[x][y]=gridboard->board[x][y];
@@ -641,11 +717,14 @@ gpointer gtk_gridboard_revert_state(GtkGridBoard *gridboard)
 
 	gtk_gridboard_clear_selections (gridboard);
 	for (x=0; x<gridboard->width; x++) {
-		for (y=0; y<gridboard->width; y++) {
-			gridboard->board[x][y]=thisstate->board[x][y];
-		}
+	  for (y=0; y<gridboard->width; y++) {
+	    if (gridboard->board[x][y] != thisstate->board[x][y]) {
+	      gridboard->board[x][y] = thisstate->board[x][y];
+	      gridboard->changed[x][y] = TRUE;
+	    }
+	  }
 	}
-
+	
 	data=thisstate->data;
 	gridboard->statelist=prevstate;
 	g_free(thisstate->board);
