@@ -21,6 +21,7 @@
 
 #include "config.h"
 #include <gnome.h>
+#include <glib/gi18n.h>
 #include <string.h>
 #include <dirent.h>
 #include <games-clock.h>
@@ -40,8 +41,14 @@
 #define GCONF_QUICKMOVES GCONF_PREFIX "quickmoves"
 #define GCONF_TILESET GCONF_PREFIX "tileset"
 #define GCONF_ANIMATE GCONF_PREFIX "animate"
-#define GCONF_SHOWGRID GCONF_PREFIX "showgrid"
-#define GCONF_FLIPFINAL GCONF_PREFIX "flipfinal"
+
+typedef struct {
+	gint black_level;
+	gint white_level;
+	gboolean animate;
+	gboolean quick_moves;
+	gchar * tile_set;
+} PropertiesData;
 
 PropertiesData _props;
 PropertiesData * props=&_props;
@@ -81,14 +88,6 @@ int props_get_black_level() {
 	return props->black_level;
 }
 
-gboolean props_get_flip_final() {
-	return props->flip_final;
-}
-
-gboolean props_get_show_grid() {
-	return props->show_grid;
-}
-
 gboolean props_get_animate() {
 	return props->animate;
 }
@@ -154,10 +153,7 @@ void load_properties (void)
 	if (props->tile_set == NULL) props->tile_set = g_strdup("classic.png");
 
 	props->animate=props_get_bool(GCONF_ANIMATE, TRUE);
-	props->show_grid=props_get_bool(GCONF_SHOWGRID, TRUE);
 	props->quick_moves=props_get_bool(GCONF_QUICKMOVES, FALSE);
-	props->flip_final=props_get_bool (GCONF_FLIPFINAL, TRUE);
-
 }
 
 static void
@@ -173,10 +169,6 @@ save_properties (void)
 	                         props->tile_set, NULL);
 	gconf_client_set_bool (get_gconf_client(), GCONF_ANIMATE,
 	                      props->animate, NULL);
-	gconf_client_set_bool (get_gconf_client(), GCONF_SHOWGRID,
-	                      props->show_grid, NULL);
-	gconf_client_set_bool (get_gconf_client(), GCONF_FLIPFINAL,
-	                       props->flip_final, NULL);
 }
 
 /* callbacks for the various check and radiobuttons */
@@ -247,7 +239,7 @@ fill_tileset_menu (void)
 /* makes a perty radio button for a particular computer level */
 static GtkWidget * add_level(GtkWidget * container, 
 		GtkWidget * prev,
-		gchar * caption,
+		const gchar * caption,
 		gint level,
 		gint piece) {
 	GtkWidget * button;
@@ -361,12 +353,12 @@ void show_properties_dialog (void) {
 	
 	vbox2 = gtk_vbox_new (FALSE, 6);
 	
-	button=add_level(vbox2, NULL, _("Human"), 	0, BLACK);
-	button=add_level(vbox2, button, _("Very easy"),	1, BLACK);
-	button=add_level(vbox2, button, _("Easy"), 	2, BLACK);
-	button=add_level(vbox2, button, _("Medium"), 	3, BLACK);
-	button=add_level(vbox2, button, _("Hard"), 	4, BLACK);
-	button=add_level(vbox2, button, _("Very hard"), 5, BLACK);
+	button=add_level(vbox2, NULL,    _("Human"), 	        0, BLACK);
+	button=add_level(vbox2, button,  _("Very easy"),	1, BLACK);
+	button=add_level(vbox2, button, Q_("gataxx|Easy"), 	2, BLACK);
+	button=add_level(vbox2, button, Q_("gataxx|Medium"), 	3, BLACK);
+	button=add_level(vbox2, button,  _("Hard"), 	        4, BLACK);
+	button=add_level(vbox2, button,  _("Very hard"),        5, BLACK);
 
 	gtk_container_add (GTK_CONTAINER (frame), vbox2);
 	
@@ -409,24 +401,6 @@ void show_properties_dialog (void) {
 	gtk_tooltips_set_tip(tooltips, button, _("Flip the pieces with some visual effects"), NULL);	
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
-	/* flip tiles button */
-	button=gtk_check_button_new_with_mnemonic(_("_Flip final results"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), props->flip_final);
-	g_signal_connect(G_OBJECT(button), "toggled",
-			G_CALLBACK(set_variable_cb), &(props->flip_final));
-	gtk_tooltips_set_tip(tooltips, button, _("Put all the white pieces at the bottom and all the black pieces on the top of the board when a game is over"), NULL);	
-	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-
-	/* show grid button */
-	button=gtk_check_button_new_with_mnemonic(_("_Show grid"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), props->show_grid);
-	g_signal_connect(G_OBJECT(button), "toggled",
-			G_CALLBACK(set_variable_cb), &(props->show_grid));
-	gtk_tooltips_set_tip(tooltips, button, _("Whether the grid should be shown"), NULL);	
-	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-
-	/* FIXME stagger flips button? */
-
 	/*  tileset select */
 	hbox=gtk_hbox_new(FALSE, 12);
 	label=gtk_label_new_with_mnemonic(_("_Tile set:"));
