@@ -41,6 +41,7 @@
 #include <config.h>
 #include <dirent.h>
 #include <string.h>
+#include <math.h>
 
 int LINES = 20;
 int COLUMNS = 11;
@@ -66,6 +67,7 @@ bool rotateCounterClockWise = true;
 #define KEY_THEME KEY_OPTIONS_DIR "/theme"
 #define KEY_STARTING_LEVEL KEY_OPTIONS_DIR "/starting_level"
 #define KEY_DO_PREVIEW KEY_OPTIONS_DIR "/do_preview"
+#define KEY_USE_TARGET KEY_OPTIONS_DIR "/use_target"
 #define KEY_RANDOM_BLOCK_COLORS KEY_OPTIONS_DIR "/random_block_colors"
 #define KEY_ROTATE_COUNTER_CLOCKWISE KEY_OPTIONS_DIR "/rotate_counter_clock_wise"
 #define KEY_LINE_FILL_HEIGHT KEY_OPTIONS_DIR "/line_fill_height"
@@ -102,6 +104,7 @@ Tetris::Tetris(int cmdlLevel):
 	timeoutId(0), 
 	onePause(false), 
 	inPlay(false),
+	useTarget(false),
 	bgimage(0),
 	setupdialog(0), 
 	cmdlineLevel(cmdlLevel), 
@@ -209,6 +212,7 @@ Tetris::Tetris(int cmdlLevel):
 
 	preview = new Preview ();
 	field = new Field();
+	field->setUseTarget (false);
 
 	initOptions ();
 
@@ -481,6 +485,8 @@ Tetris::initOptions ()
 	else
 		sound->turnOff ();
 
+	useTarget = gconfGetBoolean (gconf_client, KEY_USE_TARGET, FALSE);
+
 	do_preview = gconfGetBoolean (gconf_client, KEY_DO_PREVIEW, TRUE);
 	
 	if (preview) {
@@ -586,6 +592,19 @@ Tetris::setSelection(GtkWidget *widget, void *data)
 				 ThemeTable[t->themeno].id, NULL);
 }
 
+void 
+Tetris::setTarget (GtkWidget *widget, void *data)
+{
+	Tetris *t;
+	
+	t = (Tetris *)data;
+
+	t->useTarget = GTK_TOGGLE_BUTTON (widget)->active;
+
+	gconf_client_set_bool (t->gconf_client, KEY_USE_TARGET, 
+			       t->useTarget, NULL);
+}
+
 void
 Tetris::lineFillHeightChanged (GtkWidget *spin, gpointer data)
 {
@@ -823,6 +842,14 @@ Tetris::gameProperties(GtkAction *action, void *d)
  	gtk_box_pack_start (GTK_BOX (fvbox), t->rotate_counter_clock_wise_toggle,
 			    0, 0, 0);
 
+	t->useTargetToggle = gtk_check_button_new_with_mnemonic (_("Show where the block will _land"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (t->useTargetToggle), 
+				      t->useTarget);
+	g_signal_connect (t->useTargetToggle, "clicked",
+			  G_CALLBACK (setTarget), d);
+ 	gtk_box_pack_start (GTK_BOX (fvbox), t->useTargetToggle,
+			    0, 0, 0);
+
 	gtk_container_add (GTK_CONTAINER (frame), fvbox);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, 
 			    FALSE, FALSE, 0);
@@ -870,7 +897,7 @@ Tetris::gameProperties(GtkAction *action, void *d)
 	gtk_container_add (GTK_CONTAINER (frame), fvbox);
 
 	GtkWidget *omenu = gtk_combo_box_new_text ();
-	ThemeTableEntry *entry = ThemeTable;
+	const ThemeTableEntry *entry = ThemeTable;
 	while (entry->id) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), entry->name);
 		entry++;
@@ -938,13 +965,17 @@ Tetris::gameQuit(GtkAction *action, void *d)
 void
 Tetris::generateTimer(int level)
 {
-	g_source_remove(timeoutId);
+	g_return_if_fail (level > 0);
 
-	int intv = 1000 - 100 * (level - 1);
-	if (intv <= 0)
-		intv = 100;
+	if (timeoutId > 0)
+		g_source_remove(timeoutId);
+
+	// With 0.8, the old level 10 should be hit at about level 20.
+	int intv = (int) round (1000.0 * pow (0.8, level - 1));
+	if (intv <= 10)
+		intv = 10;
 		
-	timeoutId = g_timeout_add_full(0, intv, timeoutHandler, this, NULL);
+	timeoutId = g_timeout_add (intv, timeoutHandler, this);
 }
 
 void
@@ -955,15 +986,15 @@ Tetris::manageFallen()
 
 	int levelBefore = scoreFrame->getLevel();
 
-	int fullLines = field->checkFullLines();
-	if (fullLines > 0)
-		scoreFrame->incLines(fullLines);
-	int levelAfter = scoreFrame->getLevel();
+	int levelAfter = scoreFrame->scoreLines (field->checkFullLines());
 	if (levelAfter != levelBefore) 
 		sound->playSound (SOUND_GNOMETRIS);
 	if ((levelBefore != levelAfter) || fastFall)
 		generateTimer(levelAfter);
 	
+	if (field->isFieldEmpty ())
+		scoreFrame->scoreLastLineBonus ();
+
 	generate();
 }
 
@@ -989,13 +1020,10 @@ Tetris::timeoutHandler(void *d)
 		if (res)
 		{
 			t->manageFallen();
-			if(t->fastFall && t->inPlay) {
-				t->scoreFrame->incScore(t->fastFallPoints);
+			if (t->fastFall && t->inPlay) {
 				t->fastFall = false;
 			}
 		}
-		else if(t->fastFall)
-			++t->fastFallPoints;
 	}
 
 	return TRUE;	
@@ -1006,7 +1034,6 @@ Tetris::keyPressHandler(GtkWidget *widget, GdkEvent *event, Tetris *t)
 {
 	int keyval;
 	bool res = false;
-	int bonus;
 
 	if (t->timeoutId == 0)
 		return FALSE;
@@ -1040,16 +1067,14 @@ Tetris::keyPressHandler(GtkWidget *widget, GdkEvent *event, Tetris *t)
 	} else if (keyval == t->moveDown) {
 		if (!t->fastFall && !t->onePause) {
 			t->fastFall = true;
-			t->fastFallPoints = 0;
 			g_source_remove (t->timeoutId);
-			t->timeoutId = g_timeout_add_full(0, 10, timeoutHandler, t, NULL);
+			t->timeoutId = g_timeout_add (10, timeoutHandler, t);
 			res = true;
 		}
 	} else if (keyval == t->moveDrop) {
 		if (!t->dropBlock) {
 			t->dropBlock = true;
-			bonus = t->field->dropBlock();
-			t->scoreFrame->incScore(bonus);
+			t->field->dropBlock();
 			t->manageFallen();
 			res = TRUE;
 		}        
@@ -1341,6 +1366,8 @@ Tetris::endOfGame()
 		int pos = high_scores->add (scoreFrame->getScore());
 		high_scores->show (pos);
 	}
+
+	field->setUseTarget (false);
 }
 
 int
@@ -1368,14 +1395,15 @@ Tetris::gameNew(GtkAction *action, void *d)
 	t->scoreFrame->setLevel(level);
 	t->scoreFrame->setStartingLevel(level);
 
-	t->timeoutId = g_timeout_add_full(0, 1000 - 100 * (level - 1), timeoutHandler, t, NULL);
+	t->field->setUseTarget (t->useTarget);
+
+	t->generateTimer (level);
 	t->field->emptyField(t->line_fill_height,t->line_fill_prob);
 
-	t->scoreFrame->resetLines();
+	t->scoreFrame->resetScore();
 	t->paused = false;
 	
 	t->field->generateFallingBlock();
-	//gtk_widget_queue_draw(t->field->getWidget());
 	t->field->redraw();
 	t->preview->previewBlock(blocknr_next, rot_next, color_next);
 	gtk_widget_queue_draw(t->preview->getWidget());
